/*
 * @(#)ArcConnection.java 5.1
 *
 */

package CH.ifa.draw.cpn;

import java.awt.*;
import java.util.*;
import java.io.IOException;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.shadow.*;

public class ArcConnection extends LineConnection implements ShadowHolder {
    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -7959501008698525009L;
    private int arcConnectionSerializedDataVersion = 1;

    public static final ArcConnection NormalArc
                        = new ArcConnection(ShadowArc.ordinary);
    public static final ArcConnection TestArc
                        = new ArcConnection(ShadowArc.test);
    public static final ArcConnection ReserveArc
                        = new ArcConnection(ShadowArc.both);

    public ArcConnection() {
    }

    public ArcConnection(int arcType) {
      ArrowTip start = null, end = null;
      if (arcType == ShadowArc.both)
	start = new ArrowTip();
      if (arcType != ShadowArc.test)
	end = new ArrowTip();
      setStartDecoration(start);
      setEndDecoration(end);
    }

    public int getArcType() {
      int arrowMode = ((Integer)getAttribute("ArrowMode")).intValue();
      if (arrowMode == ARROW_TIP_NONE)
	return ShadowArc.test;
      if (arrowMode == ARROW_TIP_BOTH)
	return ShadowArc.both;
      return ShadowArc.ordinary;
    }

    public void handleConnect(Figure start, Figure end) {
        updateInscriptions();
    }

    public void handleDisconnect(Figure start, Figure end) {
      handleConnect(start,end);
   }

    public boolean canConnect(Figure start, Figure end) {
        return (start instanceof TransitionFigure
		&& end instanceof PlaceFigure
		 ||
		 start instanceof PlaceFigure
		&& end instanceof TransitionFigure);
    }

  private ShadowArc shadow = null;
  private ShadowNet net = null;

  public void buildShadow(ShadowNet net) {
    // OK: Changed the order of the following two commands.
    // I have no clue why they were swapped before.
    this.net = net;
    updateInscriptions();
  }

  public Figure locateShadow(Object shadow) {
    if (shadow==this.shadow) {
      Figure textChild=getTextChild();
      if (textChild==null) {
	return this;
      } else {
	return textChild;
      }
    } else {
      return null;
    }
  }

  public boolean isReverse() {
    return ((Integer)getAttribute("ArrowMode")).intValue() == ARROW_TIP_START;
  }

  private ShadowNetElement anyShadow(boolean end) {
    ShadowHolder holder = (ShadowHolder)(end ? endFigure() : startFigure());
    ShadowNetElement shadow = holder.getShadow();
    if (shadow == null) {
      if (net == null) {
	//System.out.println("!!! Start- or EndFigure has no shadow AND shadowNet not set!!!");
         return null;
      }
      holder.buildShadow(net);
      shadow = holder.getShadow();
    }
    return shadow;
  }

  public ShadowNetElement startShadow() {
    return anyShadow(isReverse());
  }

  public ShadowNetElement endShadow() {
    return anyShadow(!isReverse());
  }

  private CPNTextFigure getTextChild() {
    FigureEnumeration fe = children();
    while (fe.hasMoreElements()) {
      Figure child = fe.nextFigure();
      if (child instanceof CPNTextFigure) {
	CPNTextFigure textChild = (CPNTextFigure)child;
	if (textChild.getType() == CPNTextFigure.INSCRIPTION) {
	  return textChild;
	}
      }
    }
    return null;
  }

  public void updateInscriptions() {
    discardShadow();
    if (startFigure() == null || endFigure() == null)
      return;
    ShadowNetElement startShadow = startShadow(), endShadow = endShadow();
    if (startShadow == null || endShadow == null)
      return;

    String inscription = null;
    CPNTextFigure textChild=getTextChild();
    if (textChild!=null) {
      inscription = textChild.getText();
    }

    // The next two lines try to replace an empty inscription
    // by the inscription "[]". While this seems resonable at first,
    // the processing of empty inscriptions should really be left 
    // to the shadow compiler.
    // 
    // if (inscription == null || inscription.equals(""))
    //  inscription = "\"[]\"";
    shadow = new ShadowArc(startShadow,endShadow,getArcType(),inscription);
    shadow.setTrace(fTraceMode);
    try {
      shadow.syntaxCheck();
    } catch (SyntaxException e) {
	System.out.println("Exception: "+e);
    }
  }

  public ShadowNetElement getShadow() {
    return shadow;
  }

  private void discardShadow() {
    if (shadow != null) {
      shadow.discard();
      shadow = null;
      //System.err.println("arc shadow deleted");
    }
  }

  public void release() { super.release(); discardShadow(); }

  public void addChild(ChildFigure child) {
      super.addChild(child);
      if (child instanceof CPNTextFigure) {
        updateInscriptions();
      }
  }

  public void childChanged(ChildFigure child) {
      super.childChanged(child);
      if (child instanceof CPNTextFigure) {
        updateInscriptions();
      }
  }

  public void removeChild(ChildFigure child) {
      super.removeChild(child);
      if (child instanceof CPNTextFigure) {
        updateInscriptions();
      }
  }

  private boolean fTraceMode = true;

  public Object getAttribute(String name) {
    if (name.equals("TraceMode"))
      return new Boolean(fTraceMode);
    else
      return super.getAttribute(name);
  }

  public void setAttribute(String name, Object value) {
    if (name.equals("TraceMode")) {
      fTraceMode = ((Boolean)value).booleanValue();
      if (shadow != null)
	shadow.setTrace(fTraceMode);
      //      System.out.println("TraceMode "+fTraceMode+" for Arc "+this);
    } else {
      super.setAttribute(name,value);
      if (name.equals("ArrowMode")) {
          updateInscriptions();
      }
    }
  }
}
