/*
 * @(#)CPNApplication.java 5.1
 *
 */

package CH.ifa.draw.cpn;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.application.*;
import CH.ifa.draw.contrib.*;
import fs.*;
import de.renew.shadow.*;

public  class CPNApplication extends DrawApplication {

    static private final String CPNIMAGES = "/CH/ifa/draw/cpn/images/";
    private ToolButton fInscrTB, fNameTB, fDeclTB, fConnTextTB, fTextTB;

    CPNApplication() {
        this(new String[0]);
    }

    CPNApplication(String[] args) {
        super("Reference Net Workshop", ".rnw", args, CPNIMAGES+"RENEW.gif");
    }

    protected Drawing createDrawing() {
        return new CPNDrawing();
    }

    protected DrawApplication newWindow() { return new CPNApplication(); }

    /**
     * Creates the standard menus. Clients override this
     * method to add additional menus.
     */
    protected void createMenus(MenuBar mb) {
		mb.add(createFileMenu());
		mb.add(createEditMenu());
		mb.add(createAlignmentMenu());
		mb.add(createAttributesMenu());
		mb.add(createSimulationMenu()); // new!
		mb.add(createDrawingsMenu());
    }

 
    /**
     * Create the simulation menu.
     */
    protected Menu createSimulationMenu() {

	Menu menu = new Menu("Simulation");
	
    	MenuItem mi;

    	mi = new MenuItem("Run simulation", new MenuShortcut('r'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		if (!CPNDrawing.isSimulationActive()) {
		  initSimulation();
		}
		if (CPNDrawing.isSimulationActive()) {
		  showStatus("Simulation running.");
		  CPNDrawing.simulationRun();
		}
	      }
	   }
	);
	menu.add(mi);

    	mi = new MenuItem("Simulation Step", new MenuShortcut('i'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		if (!CPNDrawing.isSimulationActive()) {
		  initSimulation();
		} else {
		  if (CPNDrawing.simulationStep()) {
		    if (CPNDrawing.isSimulationActive()) {
		      showStatus("Simulation step completed.");
		    } else {
		      showStatus("Simulation step completed. "+
			"No more enabled bindings.");
		    }
		  } else {
		    showStatus("No enabled bindings found.");
		  }
		}
	      }
	   }
	);
	menu.add(mi);

    	mi = new MenuItem("Terminate simulation", new MenuShortcut('t'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		CPNDrawing.simulationTerminate();
		showStatus("Simulation terminated.");
	      }
	   }
	);
	menu.add(mi);
	
        return menu;
    }


    protected StandardDrawingView createDrawingView(int width, int height) {
        return new PageDrawingView(this, width, height);
    }


    protected Menu createFileMenu() {
	Menu menu = super.createFileMenu();
    	MenuItem mi = new MenuItem("Export PostScript...", new MenuShortcut('e'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportPostscript();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

	mi = new MenuItem("Export to Macao...");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportMacao();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

	mi = new MenuItem("Export to Woflan...");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportWoflan();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

        return menu;
    }

    protected Menu createAttributesMenu() {
        Menu menu = super.createAttributesMenu();

        CommandMenu textTypeMenu = new CommandMenu("TextType");
        textTypeMenu.add(new ChangeAttributeCommand("Label",
						    "TextType",
			           new Integer(CPNTextFigure.LABEL), this));
        textTypeMenu.add(new ChangeAttributeCommand("Inscription",
						    "TextType",
			           new Integer(CPNTextFigure.INSCRIPTION), this));
        textTypeMenu.add(new ChangeAttributeCommand("Name",
						    "TextType",
				   new Integer(CPNTextFigure.NAME), this));
	menu.add(textTypeMenu);
	

	menu.addSeparator();
        CommandMenu traceMenu = new CommandMenu("Trace");
        traceMenu.add(new ChangeAttributeCommand("on",
						 "TraceMode",
						 new Boolean(true),
						 this));
        traceMenu.add(new ChangeAttributeCommand("off",
						 "TraceMode",
						 new Boolean(false),
						 this));
	menu.add(traceMenu);
        return menu;
    }

    private SyntaxExceptionFrame syntaxFrame=null;

    void processSyntaxException(FigureException e,
				boolean displayImmediately) {
      if (syntaxFrame==null) {
	syntaxFrame=new SyntaxExceptionFrame(this);
      }

      syntaxFrame.displayException(e,displayImmediately);

      // Print the error message.
      showStatus("A syntax error occurred.");
    }

    public void doTextEdit(TextFigure textFigure, int line, int column) {
       ToolButton tb=toolButtonForTextFigure(textFigure);
       paletteUserSelected(tb);
       TextTool tt = (TextTool)tb.tool();
       tt.beginEdit(textFigure);
       if (line > 0 && column > 0) {
	    tt.setCaretPosition(line,column);
       }
    }

    public void doTextEdit(TextFigure textFigure) {
       doTextEdit(textFigure, 0,0);
    }

    boolean selectOffendingElements(FigureException e) {

      if (e.errorDrawing != null && !e.errorFigures.isEmpty()) {
	view().clearSelection();
	showDrawingViewFrame(e.errorDrawing);
	FigureEnumeration errorFigures=new FigureEnumerator(e.errorFigures);
	while (errorFigures.hasMoreElements()) {
	  view().addToSelection(errorFigures.nextFigure());
	}
	
	// Redraw the newly selected elements.	
	view().repairDamage();
	if (e.textErrorFigure != null) {
          doTextEdit(e.textErrorFigure,e.line,e.column);
        }
	return true;
      } else {
	return false;
      }
    }

    /**
     * Initialize the simulation.
     */
    void initSimulation() {
      if (!(drawing() instanceof CPNDrawing)) {
	showStatus("This drawing is not a net!");
	return;
      }

      try {
	if (((CPNDrawing)drawing()).initSimulation(this)) {
	  if (CPNDrawing.isSimulationActive()) {
	    showStatus("Simulation initialized.");
	  } else {
	    showStatus("Simulation initialized. "+
	      "No enabled bindings.");
	  }
	} else {
	  showStatus("The simulation could not be initialized.");
	}
      } catch (SyntaxException e) {
	processSyntaxException(new FigureException(e),true);
      }
    }

    /**
     * Shows a file dialog and saves drawing as postscript.
     */
    public void promptExportPostscript() {
        toolDone();
        String path = getSavePath("Export Postscript...",drawing().getName()+".ps");
        if (path != null) {
            if (!path.endsWith(".ps"))
                path += ".ps";
            exportPostscript(path);
        }
    }

    public void promptExportMacao() {
	if (drawing() instanceof CPNDrawing) {
	  toolDone();
	  String path = getSavePath("Export to Macao...",drawing().getName());
	  if (path != null) {
	    try {
	      PrintWriter p = new PrintWriter(new FileOutputStream(path));
	      ((CPNDrawing)drawing()).exportMacao(p);
	      p.close();
	    } catch (Exception e) {
	      showStatus(e.toString());
	    }
	  }
	} else {
	  showStatus("This drawing is not a net!");
	}
    }

    public void promptExportWoflan() {
	if (drawing() instanceof CPNDrawing) {
	  toolDone();
	  String path = getSavePath("Export to Woflan...",drawing().getName());
	  if (path != null) {
	    try {
	      PrintWriter p = new PrintWriter(new FileOutputStream(path));
	      ((CPNDrawing)drawing()).exportWoflan(p);
	      p.close();
	    } catch (Exception e) {
	      showStatus(e.toString());
	    }
	  }
	} else {
	  showStatus("This drawing is not a net!");
	}
    }

    private void exportPostscript(String file) {
        try {
            PostscriptWriter ps = new PostscriptWriter(file);
	    Dimension pageSize = getDrawingViewSize();
	    ps.translate(-32,-32+pageSize.height);
	    ps.setClip(64,64,pageSize.width-128,pageSize.height-128);
	    drawing().draw(ps);
            ps.dispose();
        } catch (java.io.IOException e) {
            System.out.println(e.toString());
            showStatus(e.toString());
        }
    }


    /**
     * Override to define the dimensions of the drawing view.
     */
    protected Dimension getDrawingViewSize() {
        return new Dimension(535+128,778+128);
    }

    /**
     * Gets the default size of the window.
     */
    protected Dimension defaultSize() {
        return new Dimension(640,800);
    }

    public ToolButton toolButtonForTextFigure(TextFigure figure) {
       if (figure instanceof CPNTextFigure) {
          if (figure instanceof DeclarationFigure)
             return fDeclTB;
          switch (((CPNTextFigure)figure).getType()) {
            case CPNTextFigure.INSCRIPTION: return fInscrTB;
            case CPNTextFigure.NAME: return fNameTB;
            default: { }
          }
       }
       if (figure.parent() == null) {
          return fTextTB;
       } else {
          return fConnTextTB;
       }
    }

    protected void createTools(Panel palette) {
        Tool tool;

	String compilerType=System.getProperty("de.renew.netCompiler");

        super.createTools(palette);

	palette.add(new java.awt.Canvas());
	palette.add(new java.awt.Canvas());

        tool = new PlaceFigureCreationTool(this);
        palette.add(createToolButton(CPNIMAGES+"PLACE","Place Tool", tool));

        tool = new TransitionFigureCreationTool(this);
        palette.add(createToolButton(CPNIMAGES+"TRANS","Transition Tool", tool));

        tool = new ConnectionTool(this, ArcConnection.NormalArc);
        palette.add(createToolButton(CPNIMAGES+"ARC", "Arc Tool", tool));

        tool = new ConnectionTool(this, ArcConnection.TestArc);
        palette.add(createToolButton(IMAGES+"LINE", "TestArc Tool", tool));

        tool = new ConnectionTool(this, ArcConnection.ReserveArc);
        palette.add(createToolButton(IMAGES+"CONN", "ReserveArc Tool", tool));

	CPNTextFigure inscription;
        //if ("de.renew.fs.FSNetCompiler".equals(compilerType)) {
	//  inscription = new FSFigure();
	//} else {
	  inscription = CPNTextFigure.Inscription;
	//}
        tool = new CPNTextTool(this, inscription);
        // store the inscription text tool button for immediate editing of errors:
	fInscrTB=createToolButton(CPNIMAGES+"INSCR","Inscription Tool",tool);
        palette.add(fInscrTB);

        tool = new ConnectedTextTool(this, CPNTextFigure.Name);
        fNameTB = createToolButton(CPNIMAGES+"NAME","Name Tool",tool);
        palette.add(fNameTB);

        tool = new CPNTextTool(this, new DeclarationFigure());
        fDeclTB = createToolButton(CPNIMAGES+"DECL", "Declaration Tool", tool);
        palette.add(fDeclTB);

	if (compilerType!=null && compilerType.indexOf("FS")>=0) {
	  palette.add(new java.awt.Canvas());
	  palette.add(new java.awt.Canvas());
	  
	  tool = new ConnectedTextTool(this, new ConceptFigure());
	  palette.add(createToolButton(CPNIMAGES+"TYPE",
	    "Concept Tool", tool));
	  
	  tool = new ConnectionTool(this, new IsaConnection());
	  palette.add(createToolButton(CPNIMAGES+"ARC",
	    "Is-A Tool", tool));
	  
	  tool = new PartitionCreationTool(this);
	  palette.add(createToolButton(IMAGES+"RECT",
	    "Concept Partition Tool", tool));
	}

	palette.add(new java.awt.Canvas());
	palette.add(new java.awt.Canvas());

        tool = new CreationTool(this, new RectangleFigure());
        palette.add(createToolButton(IMAGES+"RECT", "Rectangle Tool", tool));

        tool = new CreationTool(this, new RoundRectangleFigure());
        palette.add(createToolButton(IMAGES+"RRECT", "Round Rectangle Tool", tool));

        tool = new CreationTool(this, new EllipseFigure());
        palette.add(createToolButton(IMAGES+"ELLIPSE", "Ellipse Tool", tool));

        tool = new CreationTool(this, new DiamondFigure());
        palette.add(createToolButton(IMAGES+"DIAMOND", "Diamond Tool", tool));

        tool = new CreationTool(this, new TriangleFigure());
        palette.add(createToolButton(IMAGES+"TRIANGLE", "Triangle Tool", tool));

        tool = new CreationTool(this, new LineFigure());
        palette.add(createToolButton(IMAGES+"LINE", "Line Tool", tool));

        tool = new ConnectionTool(this, new LineConnection());
        palette.add(createToolButton(IMAGES+"CONN", "Connection Tool", tool));

        tool = new ConnectionTool(this, new ElbowConnection());
        palette.add(createToolButton(IMAGES+"OCONN", "Elbow Connection Tool", tool));

        tool = new ScribbleTool(this);
        palette.add(createToolButton(IMAGES+"SCRIBBL", "Scribble Tool", tool));

        tool = new PolygonTool(this);
        palette.add(createToolButton(IMAGES+"POLYGON", "Polygon Tool", tool));

        tool = new ImageFigureCreationTool(this);
        palette.add(createToolButton(CPNIMAGES+"IMAGE", "Image Tool", tool));

        tool = new TextTool(this, new TextFigure());
        fTextTB = createToolButton(IMAGES+"TEXT", "Text Tool", tool);
        palette.add(fTextTB);

        tool = new ConnectedTextTool(this, new TextFigure());
        fConnTextTB = createToolButton(IMAGES+"ATEXT","Connected Text Tool",tool);
        palette.add(fConnTextTB);

    }

    //-- main -----------------------------------------------------------

    public static void main(String[] args) {
	new CPNApplication(args);
    }
}

class SyntaxExceptionFrame {
  
  private final Frame frame;
  private final Button selectButton;
  private final TextArea textArea;
  private final CPNApplication editor;
  private FigureException e=null;
  
  SyntaxExceptionFrame(final CPNApplication editor) { 
    this.editor=editor;
    frame=new Frame("Renew: Syntax Error");
    frame.setIconImage(Toolkit.getDefaultToolkit()
      .createImage(editor.menuFrame().getIconImage().getSource()));
    textArea=new TextArea();
    textArea.setEditable(false);
    frame.add("Center",textArea);
    Panel southPanel=new Panel(new GridLayout(1,2));
    frame.add("South",southPanel);
    Button closeButton=new Button("Close");
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	frame.dispose();
      }
    });
    selectButton=new Button("Select");
    selectButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	if (e!=null && !editor.selectOffendingElements(e))
	  selectButton.setEnabled(false);
      }
    });
    southPanel.add("1",closeButton);
    southPanel.add("2",selectButton);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
	frame.dispose();
      }
    });
  }

  void displayException(FigureException e, boolean displayImmediately) {
    if (displayImmediately)
      selectButton.setEnabled(editor.selectOffendingElements(e));
    else
      selectButton.setEnabled(true);
    textArea.setText(e.getMessage());
    this.e=e;
    frame.pack();
    frame.show();
  }
}


