package CH.ifa.draw.cpn;

import java.awt.*;
import java.io.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.TextFigure;
import CH.ifa.draw.util.*;

public class CPNTextFigure extends TextFigure {

    public static final int LABEL = 0;
    public static final int INSCRIPTION = 1;
    public static final int NAME = 2;
    public static final int[] STYLE = new int[] {Font.ITALIC, Font.PLAIN,
						   Font.BOLD};

    public static final CPNTextFigure Label = new CPNTextFigure(LABEL);
    public static final CPNTextFigure Inscription = new CPNTextFigure(INSCRIPTION);
    public static final CPNTextFigure Name = new CPNTextFigure(NAME);


    private int           fType = LABEL;

    public CPNTextFigure() {
      this(LABEL);
    }

    public CPNTextFigure(int type) {
        super();
	fType = type;
	setAttribute("FontStyle",new Integer(STYLE[type]));
    }

    public int getType() {
      return fType;
    }

    public void setAttribute(String name, Object value) {
      if (name.equals("TextType")) {
	willChange();
        fType = ((Integer)value).intValue();
        super.setAttribute("FontStyle",new Integer(STYLE[fType]));
        tryUpdateParent();
      } else {
	super.setAttribute(name, value);
      }
    }

    public Object getAttribute(String name) {
        if (name.equals("TextType"))
            return new Integer(fType);
        return super.getAttribute(name);
    }

    private void tryUpdateParent() {
       if (parent() != null) {
          parent().childChanged(this);
       }
    }

    /**
     * Sets the text shown by the text figure.
     */
    public void setText(String newText) {
       super.setText(newText);
       tryUpdateParent();
    }

    public void write(StorableOutput dw) {
        super.write(dw);
	dw.writeInt(fType);
    }

    public void read(StorableInput dr) throws IOException {
        super.read(dr);
	fType = dr.readInt();
	//	fType = INSCRIPTION;
        tryUpdateParent();
    } 

  public boolean canBeParent(ParentFigure parent) {
    if (super.canBeParent(parent)) {
      switch (fType) {
	case LABEL: return true;
        case INSCRIPTION: return parent instanceof TransitionFigure ||
			         parent instanceof PlaceFigure ||
			         parent instanceof ArcConnection;
        case NAME: return parent instanceof TransitionFigure ||
		          parent instanceof PlaceFigure;
      }
    }
    return false;
  }
     

}
