/*
 * @(#)PlaceFigure.java 5.1
 *
 */

package CH.ifa.draw.cpn;

import java.awt.*;
import java.util.*;
import java.io.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import de.renew.shadow.*;

public class PlaceFigure extends EllipseFigure implements ShadowHolder {
    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -7877776241236946511L;
    private int transFigureSerializedDataVersion = 1;

    public PlaceFigure() {
        super();
    }

    public Vector handles() {
        Vector handles = super.handles();
        handles.addElement(new ArcConnectionHandle(this));
	return handles;
    }


  private ShadowPlace shadow = null;
  private Hashtable inscriptions = new Hashtable(); // ShadowInscription-Table

  public void buildShadow(ShadowNet net) {
    if (shadow == null) {
      shadow = new ShadowPlace(net);
      // OK: Added call to updateInscriptions() so that
      // inscription shadows are generated in the case that the
      // the shadow is created too late.
      updateInscriptions();
      //System.err.println("place shadow for "+this+" created!");
    }
  }

  public Figure locateShadow(Object shadow) {
    if (shadow==this.shadow) {
      return this;
    } else {
      return (Figure)inscriptions.get(shadow);
    }
  }

  private void updateInscriptions() {
    if (shadow != null) {
      discardInscriptions();
      FigureEnumeration fe = children();
      Figure child = null;
      String name = null;
      while (fe.hasMoreElements()) {
	child = fe.nextFigure();
	if (child instanceof CPNTextFigure) {
	  String inscription = ((CPNTextFigure)child).getText();
	  if (!inscription.equals("")) {
	    int type = ((CPNTextFigure)child).getType();
	    if (type == CPNTextFigure.NAME) {
	      name = inscription;
	    } else if (type == CPNTextFigure.INSCRIPTION) {
	      ShadowInscription shadowInscription=
                new ShadowInscription(shadow,inscription);
	      inscriptions.put(shadowInscription,child);
	      //              try {
	      //		shadowInscription.syntaxCheck();
	      //              } catch (SyntaxException e) {
	      //		e.printStackTrace();
		//System.out.println("Exception: "+e);
	      //              }
	    }
	  }
	}
      }
      shadow.setName(name);
      //System.err.println("place name "+name+" set");

    }
   }

  public ShadowNetElement getShadow() {
    return shadow;
  }

  public void discardInscriptions() {
    Enumeration insEnum = inscriptions.keys();
    while (insEnum.hasMoreElements())
      ((ShadowNetElement)insEnum.nextElement()).discard();
    inscriptions.clear();
    //System.err.println("all place inscriptions deleted");
  }

  public void addChild(ChildFigure child) {
      super.addChild(child);
      if (child instanceof CPNTextFigure) {
        updateInscriptions();
      }
  }

  public void childChanged(ChildFigure child) {
      super.childChanged(child);
      if (child instanceof CPNTextFigure) {
        updateInscriptions();
      }
  }

  public void removeChild(ChildFigure child) {
      super.removeChild(child);
      if (child instanceof CPNTextFigure) {
        updateInscriptions();
      }
  }

  public void release() {
    super.release();
    discardInscriptions();
    if (shadow != null) {
      shadow.discard();
      shadow = null;
      //System.err.println("place shadow deleted!");
    }
  }

   public static Dimension defaultDimension() {
      return new Dimension(20,20);
   }

}

