package de.renew.call;

import de.renew.simulator.*;
import de.renew.util.*;
import de.renew.unify.*;

public class SynchronisationRequest implements Searchable, Listener {
  // This is a convienience wrapper that allows the most simple call
  // to issue a synchronisation request.
  public static Tuple synchronize(NetInstance netInstance,
    String channelName, Tuple parameters)
  {
    SynchronisationRequest request=
      new SynchronisationRequest(netInstance,channelName,parameters);
    SearchQueue.possiblyActivated(request);
    return request.getResult();
  }

  private NetInstance netInstance;
  private String channelName;

  Tuple parameters;
  Tuple resultParameters;
  Semaphor resultSemaphor;

  boolean completed;
  final long lockOrder;
  final Lock lock;

  public SynchronisationRequest(NetInstance netInstance, String channelName,
    Tuple parameters)
  {
    this.netInstance=netInstance;
    this.channelName=channelName;

    this.parameters=parameters;
    resultParameters=null;
    resultSemaphor=new Semaphor();

    completed=false;
    lockOrder=de.renew.util.Orderer.getTicket();
    lock=new Lock();
  }

  public void notifyChange() {
    SearchQueue.possiblyActivated(this);
  }

  public Tuple getResult() {
    resultSemaphor.P();
    return resultParameters;
  }

  // I assume that I won't be searched concurrently multiple times.
  // Is that safe?
  public synchronized boolean search(Searcher searcher) {
    // Has my synchronisation been done already?
    if (completed) {
      // Yes, no need to keep me in the search queue.
      return false;
    }
    // Perpare a new search that starts from scratch.
    searcher.startSearch();

    // Require completeness of the parameters.
    Variable paramVariable=new Variable(parameters,
      searcher.currentStateRecorder);
    try {
      searcher.calcChecker.addLateVariable(paramVariable,
	searcher.currentStateRecorder);
    }
    catch (Impossible e) {
      throw new RuntimeException("Calculation checker refused to make "+
	"fresh variable late.");
    }

    // We want to find a transition within the specified
    // net instance that can fire.
    Variable netVariable=new Variable(netInstance,
      searcher.currentStateRecorder);

    // Create a binder that tries to find an appropriate channel.
    // This synchronisation is not optional.
    Binder initialBinder=
      new NetChannelBinder(netVariable,channelName,paramVariable,false,this);
    searcher.addBinder(initialBinder);

    // Create an occurrence that will inhibit multiple executions
    // of this request and that will notify me upon execution.
    Occurrence occurrence=new SynchronisationOccurrence(this);
    searcher.addOccurrence(occurrence);

    // Start the search.
    boolean result=searcher.search();

    // Undo changes to searcher state.
    searcher.removeOccurrence(occurrence);
    searcher.removeBinder(initialBinder);
    searcher.currentStateRecorder.restore();

    return result;
  }
}
