package de.renew.function;

// I wrap an object corresponding to a primitive value
// into a special layer so that primitive values and
// the corresponding objects are not confused inside
// a place instance.

public final class Value {
  public final Object value;

  public Value(Object value) {
    if (value instanceof Byte || value instanceof Short ||
      value instanceof Integer || value instanceof Long ||
      value instanceof Float || value instanceof Double ||
      value instanceof Character || value instanceof Boolean)
    {
      this.value=value;
    } else {
      if (value instanceof Value) {
	throw new RuntimeException("Tried to make a nested value. Strange.");
      } else {
	throw new RuntimeException("Tried to make an object value. Strange.");
      }
    }
  }

  public int hashCode() {
    // Add a constant to distinguish a value from its
    // wrapped object.
    return value.hashCode()+517293561;
  }

  public boolean equals(Object that) {
    if (that instanceof Value) {
      return value.equals(((Value)that).value);
    } else {
      return false;
    }
  }

  public boolean booleanValue() {
    return ((Boolean)value).booleanValue();
  }
  public char charValue() {
    return ((Character)value).charValue();
  }
  public byte byteValue() {
    return ((Number)value).byteValue();
  }
  public short shortValue() {
    return ((Number)value).shortValue();
  }
  public int intValue() {
    return ((Number)value).intValue();
  }
  public long longValue() {
    return ((Number)value).longValue();
  }
  public float floatValue() {
    return ((Number)value).floatValue();
  }
  public double doubleValue() {
    return ((Number)value).doubleValue();
  }

  public String toString() {
    return de.renew.util.Types.typify(value.getClass())+"("+value.toString()+")";
  }
}
