package de.renew.simulator;

import collections.*;
import de.renew.unify.*;

class InputArcExecutable implements EarlyExecutable {
  PlaceInstance placeInstance;
  Object token;
  boolean trace;

  InputArcExecutable(PlaceInstance placeInstance, Object token,
    boolean trace)
  {
    this.placeInstance=placeInstance;
    this.token=token;
    this.trace=trace;
  }

  public long phase() {
    return placeInstance.lockOrder;
  }

  public void lock() throws Impossible {
    placeInstance.lock.lock();
    try {
      placeInstance.reserveToken(token);
    }
    catch (Exception e) {
      placeInstance.lock.unlock();
      // No reservation actually took place.
      throw new Impossible();
    }
  }

  public void execute(Tracer tracer) {
    if (trace) {
      tracer.trace("    Removing "+token+" from "+placeInstance);
    }
    placeInstance.unreserveToken(token);
    placeInstance.removeToken(token);
    placeInstance.lock.unlock();
  }

  public void unlock() {
    placeInstance.unreserveToken(token);
    placeInstance.lock.unlock();
  }
}
