package de.renew.simulator;

import java.util.Enumeration;
import collections.*;
import de.renew.unify.*;

public class NetChannelBinder implements Binder {
  // Try bindings with all the transitions possible
  // for the given net. Usually there will be very few
  // possible transitions.
  Variable netVariable;
  String name;
  Variable downlink;
  // If true, indicates that this channel need not be used if
  // the called net has no appropriate channels at all.
  boolean isOptional;

  Listener initiator;

  public NetChannelBinder(Variable netVariable, String name, Variable downlink,
    boolean isOptional, Listener initiator)
  {
    this.downlink=downlink;
    this.name=name;
    this.netVariable=netVariable;
    this.isOptional=isOptional;
    this.initiator=initiator;
  }

  private CollectionEnumeration getTransitions(NetInstance netInstance,
    String name)
  {
    UpdatableSeq result=new LinkedList();
    
    CollectionEnumeration enum=netInstance.getNet().transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      if (transition.uplink!=null) {
	if (transition.uplink.name.equals(name)) {
	  result.insertFirst(netInstance.getInstance(transition));
	}
      }
    }
    return result.elements();
  }

  public int bindingBadness(Searcher searcher) {
    Object value=netVariable.getValue();
    if (value instanceof NetInstance) {
      NetInstance net=(NetInstance)value;
      CollectionEnumeration transitions=
        getTransitions(net,name);
      return BindingBadness.clip(transitions.numberOfRemainingElements());
    } else {
      return BindingBadness.max;
    }
  }
  public boolean bind(Searcher searcher) {
    NetInstance net=
      (NetInstance)netVariable.getValue();
    CollectionEnumeration transitions=new RandomEnumeration(
      getTransitions(net,name));

    if (isOptional && !transitions.hasMoreElements()) {
      // This channel is optional and no channel is offered at
      // the invoked net. This is too easy.
      int checkpoint=searcher.currentStateRecorder.checkpoint();
      searcher.removeBinder(this);
      boolean result=searcher.search();
      searcher.addBinder(this);
      searcher.currentStateRecorder.restore(checkpoint);
      return result;
    }

    // Either the channel is not optional, or a suitable
    // channel is provided.
    while (transitions.hasMoreElements()) {
      TransitionInstance transitionInstance=(TransitionInstance)
        transitions.nextElement();

      boolean result=false;
      int checkpoint=searcher.currentStateRecorder.checkpoint();

      try {
	TransitionOccurrence occurrence=
	  new TransitionOccurrence(transitionInstance,downlink,searcher);
	// This transition should notify the initiator
	// if it gets more possible bindings.
	if (initiator!=null) {
	  transitionInstance.listeners.includeListener(initiator);
	}
	// Now try to find a binding.
	searcher.addOccurrence(occurrence);
	searcher.removeBinder(this);
	searcher.addBinders(occurrence.getBinders());
	result=searcher.search();
	searcher.removeBinders(occurrence.getBinders());
	searcher.addBinder(this);
	searcher.removeOccurrence(occurrence);
      }
      catch (Impossible e) {
	// This is an expected exception. Simply abandon this
	// search branch and try to find another binding.
	result=false;
      }

      searcher.currentStateRecorder.restore(checkpoint);

      if (result) {
	return true;
      }
    }
    return false;
  }
}

