package de.renew.simulator;

import java.util.Hashtable;
import de.renew.unify.Impossible;
import collections.*;

public class NetInstanceImpl implements NetInstance{
  private Hashtable instanceLookup;
  private Net net;
  private int netNum;

  protected NetInstanceImpl(Net net) throws Impossible {
    if (net==null) {
      throw new Impossible();
    }
    this.net=net;
    
    netNum=net.makeNetNumber();
    
    instanceLookup=new Hashtable();

    CollectionEnumeration enum=net.places.elements();
    while (enum.hasMoreElements()) {
      Place place=(Place)enum.nextElement();
      instanceLookup.put(place,place.makeInstance(this));
    }

    enum=net.transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      instanceLookup.put(transition,new TransitionInstance(this,transition));
    }
  }

  public String toString() {
    return net.toString()+"["+netNum+"]";
  }

  // Upon creation, no transition will be checked for enabledness
  // by a searcher, because it is not contained in the
  // search queue of activated transitions. For transitions with
  // preconditions, the additional will be performed as soon
  // as the first token is put in a place, but transitions
  // without preconditions and without uplinks (if we allow
  // such beasts) must be explicitly enabled.
  public void createConfirmation(Tracer tracer) {
    // Let's print a trace message first.
    tracer.trace("    Confirming the creation of "+this);

    // Second, I inform all transitions of the confirmation.
    CollectionEnumeration enum=net.transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      getInstance(transition).createConfirmation();
    }    

    // Third, the places are notified, too.
    enum=net.places.elements();
    while (enum.hasMoreElements()) {
      Place place=(Place)enum.nextElement();
      getInstance(place).createConfirmation(tracer);
    }    
  }

  public Object getInstance(Object netObject) {
    return instanceLookup.get(netObject);
  }
  public PlaceInstance getInstance(Place place) {
    return (PlaceInstance)instanceLookup.get(place);
  }
  public TransitionInstance getInstance(Transition transition) {
    return (TransitionInstance)instanceLookup.get(transition);
  }

  public Net getNet() { return net; }
}

