package de.renew.simulator;

import de.renew.unify.*;

class SimulatorHelper {
  static NetInstance fireInitially(Net net, Searcher searcher,
    Tracer tracer, boolean asynchronous)
  {
    // Search for the initial transition and fire it.
    // We make sure that the primary instance exists
    // only after a successful initial synchronisation.

    // Create the primary instance.
    NetInstance primaryInstance;
    try {
      primaryInstance=net.makeInstance();
    }
    catch (Impossible e) {
      primaryInstance=null;
    }

    // If the creation of the primary instance succeeded,
    // Try to perform the initial synchronisation.
    ExecuteFinder finder=new ExecuteFinder();
    if (primaryInstance!=null) {
      // Fire the primary transition to get an
      // initial marking. The parameters could be
      // dependent on the parameters of this method.
      // No need to undo this action.
      Variable variable=new Variable(new Tuple(0),null);
      
      // Prepare the execution of the net. If no channel with
      // name "new" is present, simply ignore the synchronisation.
      boolean bindingFound=searcher.initiatedSearch(primaryInstance,
	"new",variable,true,finder);
      if (!bindingFound) {
	primaryInstance=null;
      }
    }

    // Execute the binding.
    if (primaryInstance!=null) {
      // Confirm the permanent activation for the
      // fully spontaneous transitions of this net. The
      // confirmation is only done if the synchronisation succeeds,
      // because conceptionally, the net instance is not created
      // otherwise. But the confirmation must be done before the
      // initial transition is executed, because another binding
      // is sought immediately.
      primaryInstance.createConfirmation(tracer);
      
      // Execute the initial transition. Its binding
      // is recorded in the finder.
      finder.fireSynchronously(tracer,asynchronous);
    }

    return primaryInstance;
  }
}
