package de.renew.simulator;

import collections.*;

public class TokenBag {
  // I prepare an array with small integers to avoid
  // creating excessive amounts of temporary integer objects.
  static final int maxStaticMult=15;
  static final Integer[] multiplicities;
  static {
    multiplicities=new Integer[maxStaticMult+1];
    for (int i=0;i<=maxStaticMult;i++) {
      multiplicities[i]=new Integer(i);
    }
  }

  private int size;
  private UpdatableMap map;
  
  TokenBag() {
    map=new HashedMap();
    size = 0;
  }

  // No synchronisation required, atomic query.
  public int getSize() { return size; }

  public synchronized int getMultiplicity(Object elem) {
    if (map.includesKey(elem)) {
      return ((Integer)map.at(elem)).intValue();
    } else {
      return 0;
    }
  }

  final static Integer toInteger(int result) {
    if (result<=maxStaticMult) {
      return multiplicities[result];
    } else {
      return new Integer(result);
    }
  }

  // Unfortunately, we have to make a copy of the
  // elements, because the hashtable might be updated
  // asynchronously.
  public synchronized CollectionEnumeration uniqueElements() {
    UpdatableSet elements=new HashedSet();
    elements.includeElements(map.keys());
    return elements.elements();
  }

  synchronized boolean includes(Object elem) {
    return map.includesKey(elem);
  }

  synchronized void add(Object elem) {
    if (map.includesKey(elem)) {
      Integer num=(Integer)map.at(elem);
      map.putAt(elem,toInteger(num.intValue()+1));
    } else {
      map.putAt(elem,toInteger(1));
    }
    ++size;
  }

  synchronized void removeOneOf(Object elem) {
    if (map.includesKey(elem)) {
      Object num=map.at(elem);
      int result=((Integer)num).intValue()-1;
      if (result==0) {
	map.removeAt(elem);
      } else {
	map.putAt(elem,toInteger(result));
      }
      --size;
    } else {
      throw new RuntimeException("Negative number of tokens detected.");
    }
  }
}
