package de.renew.simulator;

import collections.*;

/**
 * I am the token collector interface.
 * My children will collect tokens from a place and store them.
 * Upon request they will free the tokens again.
 *
 * The sole reason for my existance is that there are
 * two ways of collecting tokens: removing and testing.
 * They are implemented with different methods by the place instance
 * class, but sometimes we want a more uniform access.
 * Hence we wrap the algorithm that collects tokens in a special 
 * object, one of my children. Then one of my methods can
 * be invoked without having to make a distinction by cases.
 **/
interface TokenCollector {
  /**
   * I verify that a token of the given kond could be
   * collected, if required.
   *
   * I will not store the value anywhere, so that
   * it may change its value without notice, after I am finished.
   *
   * @param token the token to be collected
   *
   * return true, if the token would be available
   **/
  boolean containsToken(Object token);

  /**
   * I collect one token of the given kind. The argument value must not
   * change, because I might need to store it in a set.
   *
   * @param token the token to be collected
   **/
  void collectToken(Object token);
  /**
   * Free a previously collected token.
   *
   * @param token the token to be freed
   **/
  void freeToken(Object token);
}
