package de.renew.simulator;

import collections.*;
import de.renew.unify.*;

/**
 * I map local variables to their respective actual
 * values in the current context.
 *
 * @author Olaf Kummer
 **/
public class VariableMapper {
  /**
   * This hashtable does the mapping for those variables
   * that I met so far.
   **/
  UpdatableMap hashtable;

  /**
   * I (a variable mapper) am created. I map no variables so far.
   **/
  VariableMapper() {
    hashtable=new HashedMap();
  }

  /**
   * Create a new instance that already
   * maps all the local variables that are
   * known to me. Assume that I would return a certain
   * variable. If that variable is send through the copier
   * you get another variable that the new instance will return
   * after the same query.
   *
   * Put differently, you get a functional composition
   * of the copier and myself.
   *
   * @param copier
   *   the copier that maps my variables to their copies
   * @return the new variable mapper
   **/
  VariableMapper makeCopy(Copier copier) {
    VariableMapper mapper=new VariableMapper();
    CollectionEnumeration enum=hashtable.keys();
    while (enum.hasMoreElements()) {
      Object key=enum.nextElement();
      mapper.hashtable.putAt(key,
        copier.copy(hashtable.at(key)));
    }
    return mapper;
  }

  /**
   * I will map the local variable to a variable.
   * On the first call with some argument value, I return a fresh
   * variable. On successive calls with the same argument value,
   * I will always be consistent and return the original variable
   * again.
   **/
  Variable map(LocalVariable localVariable) {
    if (hashtable.includesKey(localVariable)) {
      return (Variable)hashtable.at(localVariable);
    } else {
      Variable variable=new Variable();
      hashtable.putAt(localVariable,variable);
      return variable;
    }
  }
}


