package de.renew.unify;

import collections.*;

// On the fly:
//   Check that no calculated unknown is bound.
//   Check that no calculated unknown is registered twice.
//   Check that no two calculated unknowns depend on each other.
// During the consistency check:
//   Check that all early variables have become bound.
//   Check that all late variables are either bound or calculated.

final public class CalculationChecker {
  UpdatableSet lateVariables;
  UpdatableSet earlyVariables;

  public CalculationChecker() {
    lateVariables=new HashedSet();
    earlyVariables=new HashedSet();
  }

  public void addEarlyVariable(final Variable var, StateRecorder recorder)
    throws Impossible 
  {
    if (!earlyVariables.includes(var)) {
      if (recorder!=null) {
	recorder.record(new StateRestorer() {
	  public void restore() {
	    earlyVariables.exclude(var);
	  }
	});
      }
      
      earlyVariables.include(var);
    }
  }

  public void addCalculated(Object target, Object source,
    StateRecorder recorder) throws Impossible 
  {
    Unify.unify(target,new Calculator(source,recorder),recorder);
  }

  public void addLateVariable(final Variable var, StateRecorder recorder)
    throws Impossible  
  {
    if (!lateVariables.includes(var)) {
      if (recorder!=null) {
	recorder.record(new StateRestorer() {
	  public void restore() {
	    lateVariables.exclude(var);
	  }
	});
      }
      
      lateVariables.include(var);
    }
  }
  
  private void checkLateVariables() throws Impossible {
    // All late variables must be bound or calculated.
    CollectionEnumeration enum=lateVariables.elements();
    while (enum.hasMoreElements()) {
      Variable variable=(Variable)enum.nextElement();
      if (!variable.complete) {
        throw new Impossible("There is an unassignable late variable.");
      }
    }
  }

  private void checkEarlyVariables() throws Impossible {
    // All early variables must be bound or calcuearlyd.
    CollectionEnumeration enum=earlyVariables.elements();
    while (enum.hasMoreElements()) {
      Variable variable=(Variable)enum.nextElement();
      if (!variable.bound) {
        throw new Impossible("There is an unassigned early variable.");
      }
    }
  }

  public void checkConsistency() throws Impossible {
    checkLateVariables();
    checkEarlyVariables();
  }
}
