package de.renew.unify;

import collections.*;

public final class Variable extends Carrier {
  UpdatableSet myListeners;

  public Variable() {
    super(Unify.makeUnknownArray(1),null);
    myListeners=new HashedSet();
  }
  
  public Variable(Object initValue, StateRecorder recorder) {
    super(Unify.makeInitArray(initValue),recorder);
    myListeners=new HashedSet();
  }

  // Restore the standard equals and hashCode methods.
  // They had to be changed for the carrier class
  // to detect unwanted access to incomplete objects.
  public boolean equal(Object that) {
    return this==that;
  }
  public int hashCode() {
    return System.identityHashCode(this);
  }

  public boolean isBound() {
    return bound;
  }

  void addBacklink(Carrier c, StateRecorder recorder) {
    throw new RuntimeException(
      "An object tried to be backlinked from a variable.");
  }

  void update(final Carrier old, final Object value,
    UpdatableSet listeners, StateRecorder recorder) throws Impossible
  {
    // Remember whether I was bound.
    boolean oldBound=bound;
    // Make the update.
    super.update(old,value,listeners,recorder);
    // Include my listeners in the set of listeners if
    // I have become bound.
    if (bound!=oldBound) {
      listeners.includeElements(myListeners.elements());
    }
  }

  public Object getValue() {
    return references[0];
  }

  public void addListener(final Notifiable listener, StateRecorder recorder)
    throws Impossible
  {
    if (!myListeners.includes(listener)) {
      myListeners.include(listener);
      checkRecorder(recorder);
      if (recorder!=null) {
	recorder.record(new StateRestorer() {
	  public void restore() {
	    myListeners.exclude(listener);
	  }
	});
      }
    
      // If I am already bound, i will not have another opportunity to
      // send a notification. I'll do it now, before it's too late.
      if (bound) {
	listener.boundNotify(recorder);
      }
    }
  }
}
