package fs;

import collections.*;
import java.util.*;

public interface Concept {

  /** Return the name of this Concept. */
  public String getName();

  /** Return whether this Concept is extensional. */
  public boolean isExtensional();

  /** Return whether the feature <feature> is appropriate in this Concept. 
   */
  public boolean isApprop(Integer featureCode);

  /** Return the required Type of the Value found under the given feature. */
  public Type appropType(Integer featureCode) throws NoSuchElementException;

  /** Return an Enumeration of all appropriate features. */
  public CollectionEnumeration appropFeatureCodes();

  /** Return whether this Concept is-a <that> Concept.
    * In other words, return wheter this Concept is more special than <that>
    * Concept.
    */
  public boolean isa(Concept that);

  /** Return whether this Concept is-not-a <that> Concept.
    * In other words, return wheter this Concept is incompatible with <that>
    * Concept.
    */
  public boolean isNotA(Concept that);

  public CollectionEnumeration extensionalSuperconcepts();

  public void remove();

}
