package fs;

import java.util.Vector;
import collections.HashedMap;

public class Feature {
  public String feature;
  public FeatureStructure value;
  static private Vector codeToFeat = new Vector();
  static private HashedMap featToCode = new HashedMap();
  public static final String HEAD_FEATURE = "hd";
  public static final String TAIL_FEATURE = "tl";
  public static final Integer HEAD = getCode(HEAD_FEATURE);
  public static final Integer TAIL = getCode(TAIL_FEATURE);
  
  public Feature(String feature, FeatureStructure value) {
    this.feature = feature;
    this.value = value;
  }
  
  public Feature(String feature, Object javaObject) {
    this.feature = feature;
    this.value = new JavaObjectFS(javaObject);
  }

  public Feature(String feature, Type nodetype) {
    this(feature, new FS(nodetype));
  }
  
  public Feature(String feature, Concept concept) {
    this(feature, new FS(new ConjunctiveType(concept)));
  }
  
  public Feature(String feature) { this(feature, Type.TOP); }
  
  public static Integer getCode(String feature) {
    Integer code;
    if (featToCode.includesKey(feature))
      code = (Integer)featToCode.at(feature);
    else {
      codeToFeat.addElement(feature);
      code = new Integer(codeToFeat.size()-1);
      featToCode.putAt(feature,code);
    }
    return code;
  }
  
  public static String getFeature(Integer code) {
    return (String)codeToFeat.elementAt(code.intValue());
  }
}
