package fs;

import collections.*;
import java.util.*;

public interface Type  {

  /** The most general Type. */
  public final Type TOP = new ConjunctiveType();
  public final Type ANY = new ConjunctiveType(ConceptImpl.ANY);
  public final Type LIST = new ConjunctiveType(ConceptImpl.LIST);
  public final Type ELIST = new ConjunctiveType(ConceptImpl.ELIST);
  public final Type NELIST = new ConjunctiveType(ConceptImpl.NELIST);

  /** Return the name of this Type. */
  public String getName();

  /** Return whether this Type is extensional. */
  public boolean isExtensional();

  /** Return whether the feature <feature> is appropriate in this Type. */
  public boolean isApprop(Integer featureCode);

  /** Return the required Type of the Value found under the given feature.
    * The feature has to be appropriate for this Type.
    */
  public Type appropType(Integer featureCode) throws NoSuchElementException;

  /** Return an Enumeration of all appropriate features. */
  public CollectionEnumeration appropFeatureCodes();

  /** Return whether this Type subsumes <that> Type.
    * In other words, return wheter this Type is more general than <that> Type.
    */
  public boolean subsumes(Type that);

  /** Return whether this Type is subsumed by <that> Type.
    * In other words, return wheter this Type is more special than <that>
    * Type.
    */
  public boolean subsumedBy(Type that);

  /** Return the unification of this Type and <that> Type.
    * this Type is not modified!
    */
  public Type unify(Type that) throws UnificationFailure;

  /** Return whether this Type and <that> Type are compatible.
    */
  public boolean canUnify(Type that);

  /** Look for the most general common extensional supertype of this and <that>. */
  public Type mostGeneralExtensionalSupertype(Type that);

  /** Return an Enumeration of all Concepts this Type is 
   *   composed of.
   */
  public CollectionEnumeration concepts();

}
